TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK csv_read_by_string
  VAR_EXTERNAL
    pyext_csv_update : BOOL;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR_INPUT
    FILE_NAME : STRING;
    ROW : STRING;
    COLUMN : STRING;
  END_VAR
  VAR
    OLDCODE : STRING;
    py_eval : python_eval;
    R_TRIG0 : R_TRIG;
    R_TRIG1 : R_TRIG;
    SR0 : SR;
    _TMP_CONCAT7_OUT : STRING;
    _TMP_NE16_OUT : BOOL;
    _TMP_SEL18_OUT : STRING;
    _TMP_OR15_OUT : BOOL;
    _TMP_LEFT33_OUT : STRING;
    _TMP_NE34_OUT : BOOL;
    _TMP_AND39_OUT : BOOL;
    R_TRIG2 : R_TRIG;
  END_VAR

  _TMP_CONCAT7_OUT := CONCAT('CSVRdStr("', FILE_NAME, '","', ROW, '","', COLUMN, '")');
  _TMP_NE16_OUT := NE(_TMP_CONCAT7_OUT, OLDCODE);
  _TMP_SEL18_OUT := SEL(_TMP_NE16_OUT, OLDCODE, _TMP_CONCAT7_OUT);
  OLDCODE := _TMP_SEL18_OUT;
  R_TRIG0(CLK := pyext_csv_update);
  _TMP_OR15_OUT := OR(R_TRIG0.Q, _TMP_NE16_OUT);
  py_eval(TRIG := _TMP_OR15_OUT, CODE := _TMP_CONCAT7_OUT);
  R_TRIG1(CLK := py_eval.ACK);
  _TMP_LEFT33_OUT := LEFT(py_eval.RESULT, 1);
  _TMP_NE34_OUT := NE(_TMP_LEFT33_OUT, '#');
  _TMP_AND39_OUT := AND(R_TRIG1.Q, _TMP_NE34_OUT);
  R_TRIG2(CLK := _TMP_OR15_OUT);
  SR0(S1 := _TMP_AND39_OUT, R := R_TRIG2.Q);
  ACK := SR0.Q1;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    _TMP_ADD10_OUT : UINT;
    _TMP_EQ13_OUT : BOOL;
    _TMP_SEL15_OUT : UINT;
    _TMP_AND7_OUT : BOOL;
  END_VAR

  _TMP_ADD10_OUT := ADD(COUNTER, 1);
  _TMP_EQ13_OUT := EQ(N, _TMP_ADD10_OUT);
  _TMP_SEL15_OUT := SEL(_TMP_EQ13_OUT, _TMP_ADD10_OUT, 0);
  COUNTER := _TMP_SEL15_OUT;
  _TMP_AND7_OUT := AND(_TMP_EQ13_OUT, TRIG);
  py_eval(TRIG := _TMP_AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK

FUNCTION_BLOCK csv_reload
  VAR_EXTERNAL
    pyext_csv_update : BOOL;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
  END_VAR
  VAR_INPUT
    TRIG : BOOL;
  END_VAR
  VAR
    R_TRIG0 : R_TRIG;
    F_TRIG0 : F_TRIG;
    _TMP_OR11_OUT : BOOL;
    _TMP_NOT13_OUT : BOOL;
    _TMP_MOVE12_ENO : BOOL;
    _TMP_MOVE12_OUT : BOOL;
  END_VAR

  py_eval(TRIG := TRIG, CODE := 'pyext_csv_reload()');
  R_TRIG0(CLK := py_eval.ACK);
  F_TRIG0(CLK := py_eval.ACK);
  _TMP_OR11_OUT := OR(R_TRIG0.Q, F_TRIG0.Q);
  _TMP_NOT13_OUT := NOT(R_TRIG0.Q);
  _TMP_MOVE12_OUT := MOVE(EN := _TMP_OR11_OUT, IN := _TMP_NOT13_OUT, ENO => _TMP_MOVE12_ENO);
  IF _TMP_MOVE12_ENO THEN
      pyext_csv_update := _TMP_MOVE12_OUT;
  END_IF;
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK

FUNCTION_BLOCK csv_read_by_int
  VAR_EXTERNAL
    pyext_csv_update : BOOL;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR_INPUT
    FILE_NAME : STRING;
    ROW : INT;
    COLUMN : INT;
  END_VAR
  VAR
    OLDCODE : STRING;
    py_eval0 : python_eval;
    R_TRIG1 : R_TRIG;
    R_TRIG2 : R_TRIG;
    SR0 : SR;
    _TMP_INT_TO_STRING31_OUT : STRING;
    _TMP_INT_TO_STRING2_OUT : STRING;
    _TMP_CONCAT7_OUT : STRING;
    _TMP_NE16_OUT : BOOL;
    _TMP_SEL18_OUT : STRING;
    _TMP_OR15_OUT : BOOL;
    _TMP_LEFT33_OUT : STRING;
    _TMP_NE34_OUT : BOOL;
    _TMP_AND39_OUT : BOOL;
    R_TRIG3 : R_TRIG;
  END_VAR

  _TMP_INT_TO_STRING31_OUT := INT_TO_STRING(ROW);
  _TMP_INT_TO_STRING2_OUT := INT_TO_STRING(COLUMN);
  _TMP_CONCAT7_OUT := CONCAT('CSVRdInt("', FILE_NAME, '",', _TMP_INT_TO_STRING31_OUT, ',', _TMP_INT_TO_STRING2_OUT, ')');
  _TMP_NE16_OUT := NE(_TMP_CONCAT7_OUT, OLDCODE);
  _TMP_SEL18_OUT := SEL(_TMP_NE16_OUT, OLDCODE, _TMP_CONCAT7_OUT);
  OLDCODE := _TMP_SEL18_OUT;
  R_TRIG1(CLK := pyext_csv_update);
  _TMP_OR15_OUT := OR(R_TRIG1.Q, _TMP_NE16_OUT);
  py_eval0(TRIG := _TMP_OR15_OUT, CODE := _TMP_CONCAT7_OUT);
  R_TRIG2(CLK := py_eval0.ACK);
  _TMP_LEFT33_OUT := LEFT(py_eval0.RESULT, 1);
  _TMP_NE34_OUT := NE(_TMP_LEFT33_OUT, '#');
  _TMP_AND39_OUT := AND(R_TRIG2.Q, _TMP_NE34_OUT);
  R_TRIG3(CLK := _TMP_OR15_OUT);
  SR0(S1 := _TMP_AND39_OUT, R := R_TRIG3.Q);
  ACK := SR0.Q1;
  RESULT := py_eval0.RESULT;
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK


FUNCTION_BLOCK CounterST
  VAR_INPUT
    Reset : BOOL;
  END_VAR
  VAR_OUTPUT
    Out1 : INT;
    Out0 : INT;
  END_VAR
  VAR
    Cnt0 : INT;
  END_VAR
  VAR_EXTERNAL
    ResetCounterValue : INT;
    OI_sensor_counter : LREAL;
    VFD_relay0 : DINT;
    Air_Valve_Relay0 : DINT;
    MES5_int_counter_in : LINT;
    MES5_int_counter_out : LINT;
  END_VAR

  (* this will always switch on VFD_relay0 which controls conveyor line*)
  VFD_relay0 := 1;

  IF OI_sensor_counter=2.0 THEN
    (* a rectangle was recognized thus switch ON air valve *)
    Air_Valve_Relay0 := 1;
    (* read from MES5 counter value so we continue from where we were. In this case 
    increment blow counter at MES5*)
    MES5_int_counter_in := MES5_int_counter_in + 1;
    (* set to MES5 the counter value so it is kept permanently *)
    MES5_int_counter_out := MES5_int_counter_in;
  END_IF;

  IF  OI_sensor_counter=0.0 OR OI_sensor_counter=1.0 OR OI_sensor_counter=3.0 THEN
    (* a circle / triangle or nothing was recognized thus switch OFF air valve *)
    Air_Valve_Relay0 := 0;
  END_IF;

END_FUNCTION_BLOCK

PROGRAM plc_prg
  VAR_INPUT
    Reset : BOOL;
  END_VAR
  VAR_OUTPUT
    Cnt0 : INT;
    Cnt1 : INT;
  END_VAR
  VAR
    CounterST0 : CounterST;
  END_VAR

  CounterST0(Reset := Reset);
  Cnt1 := CounterST0.Out1;
  Cnt0 := CounterST0.Out0;
END_PROGRAM


CONFIGURATION config
  VAR_GLOBAL
    ResetCounterValue : INT := 0;
    OI_sensor_counter AT %IL1.2 : LREAL;
    VFD_relay0 AT %QD0.0 : DINT;
    Air_Valve_Relay0 AT %QD0.1 : DINT;
    MES5_int_counter_in AT %IL2.2 : LINT;
    MES5_int_counter_out AT %QL2.2 : LINT;
  END_VAR
  VAR_GLOBAL
    pyext_csv_update : BOOL := TRUE;
  END_VAR

  RESOURCE resource1 ON PLC
    TASK task0(INTERVAL := T#50ms,PRIORITY := 0);
    PROGRAM instance0 WITH task0 : plc_prg;
  END_RESOURCE
END_CONFIGURATION
