##############################################################################
#
# Copyright (c) 2002-2023 Nexedi SA and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
##############################################################################
import json
from Products.ERP5Type.tests.ERP5TypeTestCase import ERP5TypeTestCase

class TestOPCUADocumentDataInestgestion(ERP5TypeTestCase):

  """
  Test Class for OPC UA Document implementation.
  """

  def getTitle(self):
    return "OPC UA Document"

  def test_01_SetAndGet(self):
    """
    Simulate OPC UA client -> server integration.
    """
    # create placeholder
    default_id = "1"
    default_opcua_document = self.portal.opcua_document_module.get(default_id, None)
    if default_opcua_document is None:
      default_opcua_document = self.portal.opcua_document_module.newContent(portal_type = "OPCUA Document",
                                                                            id = default_id)
    self.tic()

    # cleanup
    default_opcua_document.setNodeDict({})
    self.tic()

    # test HTTP GET returns empty dict
    result_dict = self.portal.ERP5Site_handleOPCUARequest(http_method="GET", 
                                                          default_opcua_document=default_opcua_document)
    self.assertEqual(json.dumps({}), result_dict)

    # test setting over a client
    node_id = "n1=1;i=810"
    node_val = 10
    type_data = "DataChangeNotification(<asyncua.common.subscription.SubscriptionItemData object at 0x7f1761723a60>, MonitoredItemNotification(ClientHandle=202, Value=DataValue(Value=Variant(Value=1112, VariantType=<VariantType.Int64: 8>, Dimensions=None, is_array=False), StatusCode_=StatusCode(value=0), SourceTimestamp=None, ServerTimestamp=datetime.datetime(2023, 9, 5, 13, 23, 39, 285445), SourcePicoseconds=None, ServerPicoseconds=None)))"
    opcua_structure = {"node": node_id,
                       "val":node_val,
                       "data": type_data}
    result_json = self.portal.ERP5Site_handleOPCUARequest(http_method="POST", 
                                                         default_opcua_document=default_opcua_document,
                                                         data = json.dumps(opcua_structure))
    # convert from JSON string to dict
    result_dict = json.loads(result_json)
    self.assertEqual({node_id: node_val}, result_dict)

    # test getting over a client
    result_json = self.portal.ERP5Site_handleOPCUARequest(http_method="GET", 
                                                          default_opcua_document=default_opcua_document)
    result_dict = json.loads(result_json)
    self.assertEqual({node_id: node_val}, result_dict)

    # test UPDATE
    node_val = 100
    opcua_structure = {"node": node_id,
                       "val":node_val,
                       "data": type_data}
    result_json = self.portal.ERP5Site_handleOPCUARequest(http_method="POST", 
                                                         default_opcua_document=default_opcua_document,
                                                         data = json.dumps(opcua_structure))
    result_dict = json.loads(result_json)
    self.assertEqual({node_id: node_val}, result_dict)